type line_mode = One | Two
let line_mode = ref One
let cgrom = Cgrom.parse ()
let ddram = Array.make 104 ' '
let ddram_addr = ref 0

let set_addr addr =
  let new_addr =
    match !line_mode with
      | One ->
	  begin match addr with
	    | -1 -> 79
	    | 80 | 128 -> 0
	    | _ -> addr
	  end
      | Two ->
	  begin match addr with
	    | -1 -> 103
	    | 40 -> 64
	    | 104 | 128 -> 0
	    | _ -> addr
	  end
  in
    assert (new_addr >= 0 && new_addr < 128);
    ddram_addr := new_addr;
;;

let incr_addr () = set_addr (!ddram_addr + 1);;

let decr_addr () = set_addr (!ddram_addr - 1);;

let convert_addr addr =
  assert (addr >= 0 && addr < 128);
  match !line_mode with
    | One -> if addr < 80 then addr else invalid_arg "convert_addr"
    | Two ->
	if addr < 40 then addr
	else if addr < 64 then addr + 64 - 40
	else if addr < 104 then addr
	else if addr < 112 then addr - 104
	else if addr < 120 then addr - 112
	else addr - 120
;;

let write c =
  try ddram.(convert_addr !ddram_addr) <- c;
  with Invalid_argument "convert_addr" -> ();
;;

let read () =
  try ddram.(convert_addr !ddram_addr);
  with Invalid_argument "convert_addr" -> ' ';
;;

let fill c =
  Array.fill ddram 0 (Array.length ddram) c;
;;

let rotate_left () =
  let ddram103 = ddram.(103) in
    for i = 102 downto 0 do
      ddram.(succ i) <- ddram.(i);
    done;
    ddram.(0) <- ddram103;
;;

let rotate_right () =
  let ddram0 = ddram.(0) in
    for i = 0 to 102 do
      ddram.(i) <- ddram.(succ i);
    done;
    ddram.(103) <- ddram0;
;;

let get_addr () = !ddram_addr;;

let set_two_line_mode b =
  line_mode := if b then Two else One;
;;

let to_matrix m =
  for i = 0 to 39 do
    m.(i).(0) <- cgrom.(int_of_char ddram.(i));
  done;
  if !line_mode = Two then
    for i = 0 to 39 do
      m.(i).(1) <- cgrom.(int_of_char ddram.(i + 64));
    done
  else
    for i = 0 to 39 do
      m.(i).(1) <- cgrom.(int_of_char ' ');
    done
;;
